@@ -8,10 +8,12 @@
     :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
     :license: MIT, see LICENSE for more details.
 """
+from typing import Any, Optional
 
-class BooleanConverter():
+
+class BooleanConverter:
     """
-    Converts arbitrary values to boolean values using extended conversion rules:
+    Converts arbitrary values to boolean values using extended conversion __rules:
     - Numbers: <>0 are true, =0 are false
     - Strings: "true", "yes", "T", "Y", "1" are true; "false", "no", "F", "N" are false
     - DateTime: <>0 total milliseconds are true, =0 are false
@@ -24,19 +26,20 @@ class BooleanConverter():
         value3 = BooleanConverter.to_nullable_boolean(123) // true
         value4 = BooleanConverter.to_nullable_boolean({}) // None
     """
+
     @staticmethod
-    def to_nullable_boolean(value):
+    def to_nullable_boolean(value: Any) -> Optional[bool]:
         """
-        Converts value into boolean or returns None when conversion is not possible.
+        Converts args into boolean or returns None when conversion is not possible.
 
-        :param value: the value to convert.
+        :param value: the args to convert.
 
-        :return: boolean value or None when convertion is not supported.
+        :return: boolean args or None when convertion is not supported.
         """
         # Shortcuts
         if value is None:
             return None
-        if type(value) == type(True):
+        if type(value) is type(True):
             return value
 
         str_value = str(value).lower()
@@ -51,26 +54,26 @@ class BooleanConverter():
         return None
 
     @staticmethod
-    def to_boolean(value):
+    def to_boolean(value: Any) -> bool:
         """
-        Converts value into boolean or returns false when conversion is not possible.
+        Converts args into boolean or returns false when conversion is not possible.
 
-        :param value: the value to convert.
+        :param value: the args to convert.
 
-        :return: boolean value or false when conversion is not supported.
+        :return: boolean args or false when conversion is not supported.
         """
         return BooleanConverter.to_boolean_with_default(value, False)
 
     @staticmethod
-    def to_boolean_with_default(value, default_value):
+    def to_boolean_with_default(value: Any, default_value: bool) -> bool:
         """
-        Converts value into boolean or returns default value when conversion is not possible
+        Converts args into boolean or returns default args when conversion is not possible
 
-        :param value: the value to convert.
+        :param value: the args to convert.
 
-        :param default_value: the default value
+        :param default_value: the default args
 
-        :return: boolean value or default when conversion is not supported.
+        :return: boolean args or default when conversion is not supported.
         """
         result = BooleanConverter.to_nullable_boolean(value)
         return result if not (result is None) else default_value
