@@ -8,12 +8,14 @@
     :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
     :license: MIT, see LICENSE for more details.
 """
+from typing import Any, Optional, Union
+
 from pip_services3_commons.convert.DoubleConverter import DoubleConverter
 
 
-class FloatConverter():
+class FloatConverter:
     """
-    Converts arbitrary values into float using extended conversion rules:
+    Converts arbitrary values into float using extended conversion __rules:
     - Strings are converted to float values
     - DateTime: total number of milliseconds since unix epoсh
     - Boolean: 1 for true and 0 for false
@@ -27,37 +29,37 @@ class FloatConverter():
     """
 
     @staticmethod
-    def to_nullable_float(value):
+    def to_nullable_float(value: Any) -> Optional[float]:
         """
-        Converts value into float or returns null when conversion is not possible.
+        Converts args into float or returns null when conversion is not possible.
 
-        :param value: the value to convert.
+        :param value: the args to convert.
 
-        :return: float value or null when conversion is not supported.
+        :return: float args or null when conversion is not supported.
         """
         # Shortcuts
         return DoubleConverter.to_nullable_double(value)
 
     @staticmethod
-    def to_float(value):
+    def to_float(value: Any) -> float:
         """
-        Converts value into float or returns 0 when conversion is not possible.
+        Converts args into float or returns 0 when conversion is not possible.
 
-        :param value: the value to convert.
+        :param value: the args to convert.
 
-        :return: float value or 0 when conversion is not supported.
+        :return: float args or 0 when conversion is not supported.
         """
         return DoubleConverter.to_double(value)
 
     @staticmethod
-    def to_float_with_default(value, default_value):
+    def to_float_with_default(value: Any, default_value: float) -> float:
         """
-        Converts value into float or returns default when conversion is not possible.
+        Converts args into float or returns default when conversion is not possible.
 
-        :param value: the value to convert.
+        :param value: the args to convert.
 
-        :param default_value: the default value.
+        :param default_value: the default args.
 
-        :return: float value or default value when conversion is not supported.
+        :return: float args or default args when conversion is not supported.
         """
         return DoubleConverter.to_double_with_default(value, default_value)
