@@ -10,6 +10,7 @@
 """
 
 from datetime import datetime
+from typing import Any, Optional
 
 from .ArrayConverter import ArrayConverter
 from .DateTimeConverter import DateTimeConverter
@@ -21,11 +22,12 @@ from .StringConverter import StringConverter
 from .TypeCode import TypeCode
 
 
+
 class TypeConverter:
     """
     Converts arbitrary values into objects specific by TypeCodes.
     For each TypeCode this class calls corresponding converter which applies
-    extended conversion rules to convert the values.
+    extended conversion __rules to convert the values.
 
     Example:
 
@@ -37,11 +39,11 @@ class TypeConverter:
     """
 
     @staticmethod
-    def to_type_code(value):
+    def to_type_code(value: Any) -> TypeCode:
         """
-        Gets :class:`TypeCode <pip_services3_commons.convert.TypeCode.TypeCode>` for specific value.
+        Gets :class:`TypeCode <pip_services3_commons.convert.TypeCode.TypeCode>` for specific args.
 
-        :param value: value whose TypeCode is to be resolved.
+        :param value: args whose TypeCode is to be resolved.
 
         :return: the TypeCode that corresponds to the passed object's type.
         """
@@ -61,13 +63,13 @@ class TypeConverter:
             return TypeCode.Boolean
         elif value is int or issubclass(value, int):
             return TypeCode.Integer
-        # elif value is long:
+        # elif args is long:
         #     return TypeCode.Long
         elif value is float or issubclass(value, float):
             return TypeCode.Float
         elif value is str or issubclass(value, str):
             return TypeCode.String
-        # elif value is unicode:
+        # elif args is unicode:
         #     return TypeCode.String
         elif value is datetime or issubclass(value, datetime):
             return TypeCode.DateTime
@@ -77,15 +79,15 @@ class TypeConverter:
         return TypeCode.Object
 
     @staticmethod
-    def to_nullable_type(value_type, value):
+    def to_nullable_type(value_type: TypeCode, value: Any) -> Any:
         """
-        Converts value into an object type specified by Type Code or returns null when conversion is not possible.
+        Converts args into an object type specified by Type Code or returns null when conversion is not possible.
 
-        :param value_type: the TypeCode for the data type into which 'value' is to be converted.
+        :param value_type: the TypeCode for the data type into which 'args' is to be converted.
 
-        :param value: the value to convert.
+        :param value: the args to convert.
 
-        :return: object value of type corresponding to TypeCode, or null when conversion is not supported.
+        :return: object args of type corresponding to TypeCode, or null when conversion is not supported.
         """
         value_type = TypeConverter.to_type_code(value_type) if isinstance(value_type, type) else value_type
 
@@ -115,22 +117,22 @@ class TypeConverter:
         return value
 
     @staticmethod
-    def to_type(value_type, value):
+    def to_type(value_type: TypeCode, value: Any) -> Any:
         """
-        Converts value into an object type specified by Type Code or returns type default when conversion is not possible.
+        Converts args into an object type specified by Type Code or returns type default when conversion is not possible.
 
-        :param value_type: the TypeCode for the data type into which 'value' is to be converted.
+        :param value_type: the TypeCode for the data type into which 'args' is to be converted.
 
-        :param value: the value to convert.
+        :param value: the args to convert.
 
-        :return: object value of type corresponding to TypeCode, or type default when conversion is not supported.
+        :return: object args of type corresponding to TypeCode, or type default when conversion is not supported.
         """
         # Convert to the specified type
         result = TypeConverter.to_nullable_type(value_type, value)
         if result is not None:
             return result
 
-        # Define and return default value based on type
+        # Define and return default args based on type
         result_type = TypeConverter.to_type_code(value_type)
         if result_type == TypeCode.String:
             return ''
@@ -154,29 +156,29 @@ class TypeConverter:
             return None
 
     @staticmethod
-    def to_type_with_default(value_type, value, default_value):
+    def to_type_with_default(value_type: TypeCode, value: Any, default_value: Any) -> Any:
         """
-        Converts value into an object type specified by Type Code or returns default value when conversion is not possible.
+        Converts args into an object type specified by Type Code or returns default args when conversion is not possible.
 
-        :param value_type: the :class:`TypeCode <pip_services3_commons.convert.TypeCode.TypeCode>` for the data type into which 'value' is to be converted.
+        :param value_type: the :class:`TypeCode <pip_services3_commons.convert.TypeCode.TypeCode>` for the data type into which 'args' is to be converted.
 
-        :param value: the value to convert.
+        :param value: the args to convert.
 
-        :param default_value: the default value to return if conversion is not possible (returns None).
+        :param default_value: the default args to return if conversion is not possible (returns None).
 
-        :return: object value of type corresponding to :class:`TypeCode <pip_services3_commons.convert.TypeCode.TypeCode>`, or default value when conversion is not supported.
+        :return: object args of type corresponding to :class:`TypeCode <pip_services3_commons.convert.TypeCode.TypeCode>`, or default args when conversion is not supported.
         """
         result = TypeConverter.to_nullable_type(value_type, value)
         return result if not (result is None) else default_value
 
     @staticmethod
-    def to_string(typ):
+    def to_string(typ: TypeCode) -> str:
         """
         Converts a :class:`TypeCode <pip_services3_commons.convert.TypeCode.TypeCode>` into its string name.
 
         :param typ: the TypeCode to convert into a string.
 
-        :return: the name of the TypeCode passed as a string value.
+        :return: the name of the TypeCode passed as a string args.
         """
         if typ is None:
             return "unknown"
