@@ -3,45 +3,46 @@
     pip_services3_commons.errors.ApplicationExceptionFactory
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     
-    Application exception factory implementation
+    Application error factory implementation
     
     :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
     :license: MIT, see LICENSE for more details.
 """
-
-from .ErrorCategory import ErrorCategory
+from pip_services3_commons.errors.ErrorDescription import ErrorDescription
 from .ApplicationException import ApplicationException
-from .UnknownException import UnknownException
-from .InternalException import InternalException
-from .ConfigException import ConfigException
-from .InvalidStateException import InvalidStateException
-from .ConnectionException import ConnectionException
-from .InvocationException import InvocationException
-from .FileException import FileException
 from .BadRequestException import BadRequestException
+from .ConfigException import ConfigException
+from .ConflictException import ConflictException
+from .ConnectionException import ConnectionException
+from .ErrorCategory import ErrorCategory
+from .FileException import FileException
+from .InternalException import InternalException
+from .InvocationException import InvocationException
 from .NotFoundException import NotFoundException
 from .UnauthorizedException import UnauthorizedException
-from .ConflictException import ConflictException
+from .UnknownException import UnknownException
 from .UnsupportedException import UnsupportedException
 
-class ApplicationExceptionFactory(object):
+
+class ApplicationExceptionFactory:
     """
     Factory to recreate exceptions from :class:`ErrorDescription <pip_services3_commons.errors.ErrorDescription.ErrorDescription>` values passed through the wire.
     """
+
     @staticmethod
-    def create(description):
+    def create(description: ErrorDescription) -> ApplicationException:
         """
         Recreates ApplicationException object from serialized ErrorDescription.
 
-        It tries to restore original exception type using type or error category fields.
+        It tries to restore original error type using type or error category fields.
 
         :param description: a serialized error description received as a result of remote call
 
         :return: ApplicationException object from serialized ErrorDescription.
         """
-        error = None
-        
-        # Create well-known exception type based on error category
+        error: ApplicationException
+
+        # Create well-known error type based on error category
         if ErrorCategory.Unknown == description.category:
             error = UnknownException(description.correlation_id, description.code, description.message)
         elif ErrorCategory.Internal == description.category:
@@ -68,10 +69,10 @@ class ApplicationExceptionFactory(object):
             error = UnknownException()
             error.category = description.category
             error.status = description.status
-        
+
         # Fill error with details
         error.details = description.details
         error.set_cause_string(description.cause)
         error.set_stack_trace_string(description.stack_trace)
-        
+
         return error
