@@ -11,8 +11,9 @@
 import inspect
 import time
 from threading import Thread, Event, Lock
+from typing import Callable, Any, Optional
 
-from pip_services3_commons.run import Parameters
+from pip_services3_commons.run import Parameters, INotifiable
 from .IClosable import IClosable
 
 
@@ -32,7 +33,7 @@ class Timer(Thread):
 
     def stop(self):
         if self.is_alive():
-            # set event to signal thread to terminate
+            # set event_name to signal thread to terminate
             self._event.set()
             # block calling thread until thread really has terminated
             self.join()
@@ -46,25 +47,25 @@ class FixedRateTimer(IClosable):
     and is often used by Pip.Services toolkit to perform periodic processing and cleanup in microservices.
     """
 
-    _lock = None
+    __lock = None
 
-    def __init__(self, task_or_object=None, interval=None, delay=None):
+    def __init__(self, task_or_object: Any = None, interval: int = None, delay: int = None):
         """
         Creates new instance of the timer and sets its values.
 
-        :param task: (optional) a Notifiable object or callback function to call when timer is triggered.
+        :param task_or_object: (optional) a Notifiable object or callback function to call when timer is triggered.
 
-        :param interval: (optional) an _interval to trigger timer in milliseconds.
+        :param interval: (optional) an __interval to trigger timer in milliseconds.
 
-        :param delay: (optional) a _delay before the first triggering in milliseconds.
+        :param delay: (optional) a __delay before the first triggering in milliseconds.
         """
-        self._lock = Lock()
-        self._started = False
-        self._task = None
-        self._callback = None
-        self._timer = None
-        self._interval = None
-        self._delay = None
+        self.__lock = Lock()
+        self.__task: INotifiable = None
+        self.__callback: Callable = None
+        self.__delay: int = None
+        self.__interval: int = None
+        self.__timer: Any = None
+        self.__started: bool = False
 
         if inspect.isclass(task_or_object) and hasattr(task_or_object, 'notify') and inspect.isfunction(
                 task_or_object.notify):
@@ -75,104 +76,104 @@ class FixedRateTimer(IClosable):
         self.set_interval(interval)
         self.set_delay(delay)
 
-    def get_task(self):
+    def get_task(self) -> INotifiable:
         """
         Gets the INotifiable object that receives notifications from this timer.
 
         :return: the INotifiable object or null if it is not set.
         """
-        return self._task
+        return self.__task
 
-    def set_task(self, value):
-        self._task = value
-        self._callback = self._timer_callback
+    def set_task(self, value: INotifiable):
+        self.__task = value
+        self.__callback = self.__timer_callback
 
-    def get_callback(self):
+    def get_callback(self) -> Callable:
         """
         Gets the callback function that is called when this timer is triggered.
 
         :return: the callback function or null if it is not set.
         """
-        return self._callback
+        return self.__callback
 
-    def set_callback(self, value):
+    def set_callback(self, value: Callable):
         """
         Sets the callback function that is called when this timer is triggered.
 
         :param value: the callback function to be called.
         """
-        self._callback = value
-        self._task = None
+        self.__callback = value
+        self.__task = None
 
-    def get_delay(self):
+    def get_delay(self) -> int:
         """
         Gets initial delay before the timer is triggered for the first time.
 
         :return: the delay in milliseconds.
         """
-        return self._delay
+        return self.__delay
 
-    def set_delay(self, value):
+    def set_delay(self, value: int):
         """
         Sets initial delay before the timer is triggered for the first time.
         :param value: a delay in milliseconds.
         """
-        self._delay = value
+        self.__delay = value
 
-    def get_interval(self):
+    def get_interval(self) -> int:
         """
         Gets periodic timer triggering interval.
 
         :return: the interval in milliseconds
         """
-        return self._interval
+        return self.__interval
 
-    def set_interval(self, value):
+    def set_interval(self, value: int):
         """
         Sets periodic timer triggering interval.
 
         :param value: an interval in milliseconds.
         """
-        self._interval = value
+        self.__interval = value
 
-    def is_started(self):
+    def is_started(self) -> bool:
         """
         Checks if the timer is started.
 
         :return: true if the timer is started and false if it is stopped.
         """
-        return self._timer is not None
+        return self.__timer is not None
 
     def start(self):
         """
         Starts the timer.
-        Initially the timer is triggered after _delay.
-        After that it is triggered after _interval until it is stopped.
+        Initially the timer is triggered after __delay.
+        After that it is triggered after __interval until it is stopped.
         """
-        self._lock.acquire()
+        self.__lock.acquire()
         try:
             # Stop previously set timer
-            if not (self._timer is None):
-                self._timer.stop()
-                self._timer = None
+            if not (self.__timer is None):
+                self.__timer.stop()
+                self.__timer = None
 
-            if self._interval is None or self._interval <= 0:
+            if self.__interval is None or self.__interval <= 0:
                 return
 
-            delay = max(0, self._delay - self._interval)
+            delay = max(0, self.__delay - self.__interval)
 
             # Set a new timer
-            self._timer = Timer(self._interval / 1000, delay / 1000, self._callback)
-            self._timer.start()
+            self.__timer = Timer(self.__interval / 1000, delay / 1000, self.__callback)
+            self.__timer.start()
 
-            # Set _started flag
-            self._started = True
+            # Set __started flag
+            self.__started = True
         finally:
-            self._lock.release()
+            self.__lock.release()
 
-    def _timer_callback(self):
+    def __timer_callback(self):
         try:
-            self._task.notify("pip-commons-timer", Parameters())
+            self.__task.notify("pip-commons-timer", Parameters())
         except:
             # Ignore or better log
             pass
@@ -181,22 +182,22 @@ class FixedRateTimer(IClosable):
         """
         Stops the timer.
         """
-        self._lock.acquire()
+        self.__lock.acquire()
         try:
             # Stop the timer
-            if not (self._timer is None):
-                self._timer.stop()
-                self._timer = None
+            if not (self.__timer is None):
+                self.__timer.stop()
+                self.__timer = None
 
-            # Unset _started flag
-            self._started = False
+            # Unset __started flag
+            self.__started = False
         finally:
-            self._lock.release()
+            self.__lock.release()
 
-    def close(self, correlation_id):
+    def close(self, correlation_id: Optional[str]):
         """
         Closes the timer.
-        This is required by :class:`IClosable <pip_services3_commons.run.IClosable.IClosable>` interface,
+        This is __required by :class:`IClosable <pip_services3_commons.run.IClosable.IClosable>` interface,
         but besides that it is identical to stop().
 
         :param correlation_id: (optional) transaction id to trace execution through call chain.
