@@ -8,33 +8,38 @@
     :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
     :license: MIT, see LICENSE for more details.
 """
+from typing import Any
 
-from ..data.AnyValueMap import AnyValueMap
+from pip_services3_commons.config import ConfigParams
+from pip_services3_commons.run import Parameters
 from ..convert.JsonConverter import JsonConverter
+from ..data.AnyValueMap import AnyValueMap
+from ..reflect.ObjectWriter import ObjectWriter
 from ..reflect.RecursiveObjectReader import RecursiveObjectReader
 from ..reflect.RecursiveObjectWriter import RecursiveObjectWriter
-from ..reflect.ObjectWriter import ObjectWriter
+
 
 class Parameters(AnyValueMap):
     """
     Contains map with execution parameters.
 
     In general, this map may contain non-serializable values.
-    And in contrast with other maps, its getters and setters
+    And in contrast with obj maps, its getters and setters
     support dot notation and able to access properties in the entire object graph.
 
     This class is often use to pass execution and notification
     arguments, and parameterize classes before execution.
     """
-    def __init__(self, map = None):
+
+    def __init__(self, map: Any = None):
         """
-        Creates a new instance of the map and assigns its value.
+        Creates a new instance of the map and assigns its args.
 
         :param map:(optional) values to initialize this map.
         """
         super(Parameters, self).__init__(map)
 
-    def get(self, key):
+    def get(self, key: str) -> Any:
         """
         Gets a map element specified by its key.
 
@@ -43,7 +48,7 @@ class Parameters(AnyValueMap):
 
         :param key: a key of the element to get.
 
-        :return: the value of the map element.
+        :return: the args of the map element.
         """
         if key is None or key == '':
             return None
@@ -52,16 +57,16 @@ class Parameters(AnyValueMap):
         else:
             return super(Parameters, self).get(key)
 
-    def put(self, key, value):
+    def put(self, key: str, value: Any) -> Any:
         """
-        Puts a new value into map element specified by its key.
+        Puts a new args into map element specified by its key.
 
         The key can be defined using dot notation
         and allows to recursively access elements of elements.
 
         :param key: a key of the element to put.
 
-        :param value: a new value for map element.
+        :param value: a new args for map element.
         """
         if key is None or key == '':
             return None
@@ -72,42 +77,42 @@ class Parameters(AnyValueMap):
             self[key] = value
             return value
 
-    def get_as_nullable_parameters(self, key):
+    def get_as_nullable_parameters(self, key: str) -> 'Parameters':
         """
         Converts map element into an Parameters or returns null if conversion is not possible.
 
         :param key: a key of element to get.
 
-        :return: Parameters value of the element or null if conversion is not supported.
+        :return: Parameters args of the element or null if conversion is not supported.
         """
         value = self.get_as_nullable_map(key)
         return Parameters(value) if not (value is None) else None
 
-    def get_as_parameters(self, key):
+    def get_as_parameters(self, key: str) -> 'Parameters':
         """
         Converts map element into an Parameters or returns empty Parameters if conversion is not possible.
 
         :param key: a key of element to get.
 
-        :return: Parameters value of the element or empty Parameters if conversion is not supported.
+        :return: Parameters args of the element or empty Parameters if conversion is not supported.
         """
         value = self.get_as_map(key)
         return Parameters(value)
 
-    def get_as_parameters_with_default(self, key, default_value):
+    def get_as_parameters_with_default(self, key: str, default_value: 'Parameters') -> 'Parameters':
         """
-        Converts map element into an Parameters or returns default value if conversion is not possible.
+        Converts map element into an Parameters or returns default args if conversion is not possible.
 
         :param key: a key of element to get.
 
-        :param default_value: the default value
+        :param default_value: the default args
 
-        :return: Parameters value of the element or default value if conversion is not supported.
+        :return: Parameters args of the element or default args if conversion is not supported.
         """
         result = self.get_as_nullable_parameters(key)
         return result if not (result is None) else default_value
 
-    def contains_key(self, key):
+    def contains_key(self, key: str) -> bool:
         """
         Checks if this map contains an element with specified key.
 
@@ -120,7 +125,7 @@ class Parameters(AnyValueMap):
         """
         return RecursiveObjectReader.has_property(self, key)
 
-    def override(self, parameters, recursive = False):
+    def override(self, parameters: Parameters, recursive: bool = False) -> 'Parameters':
         """
         Overrides parameters with new values from specified Parameters and returns a new Parameters object.
 
@@ -141,7 +146,7 @@ class Parameters(AnyValueMap):
 
         return result
 
-    def set_defaults(self, default_values, recursive = False):
+    def set_defaults(self, default_values: 'Parameters', recursive: bool = False) -> 'Parameters':
         """
         Set default values from specified Parameters and returns a new Parameters object.
 
@@ -162,18 +167,18 @@ class Parameters(AnyValueMap):
 
         return result
 
-    def assign_to(self, value):
+    def assign_to(self, value: Any):
         """
-        Assigns (copies over) properties from the specified value to this map.
+        Assigns (copies over) properties from the specified args to this map.
 
-        :param value: value whose properties shall be copied over.
+        :param value: args whose properties shall be copied over.
         """
         if value is None or len(self) == 0:
             return
 
         RecursiveObjectWriter.copy_properties(value, self)
-        
-    def pick(self, *props):
+
+    def pick(self, *props: str) -> 'Parameters':
         """
         Picks select parameters from this Parameters and returns them as a new Parameters object.
 
@@ -187,7 +192,7 @@ class Parameters(AnyValueMap):
                 result.put(prop, self.get(prop))
         return result
 
-    def omit(self, *props):
+    def omit(self, *props: str) -> 'Parameters':
         """
         Omits selected parameters from this Parameters and returns the rest as a new Parameters object.
 
@@ -200,7 +205,7 @@ class Parameters(AnyValueMap):
             del result[prop]
         return result
 
-    def to_json(self):
+    def to_json(self) -> str:
         """
         Converts this map to JSON object.
 
@@ -209,11 +214,11 @@ class Parameters(AnyValueMap):
         return JsonConverter.to_json(self)
 
     @staticmethod
-    def from_value(value):
+    def from_value(value: Any) -> 'Parameters':
         """
-        Creates a new Parameters object filled with key-value pairs from specified object.
+        Creates a new Parameters object filled with key-args pairs from specified object.
 
-        :param value: an object with key-value pairs used to initialize a new Parameters.
+        :param value: an object with key-args pairs used to initialize a new Parameters.
 
         :return: a new Parameters object.
         """
@@ -221,9 +226,9 @@ class Parameters(AnyValueMap):
         return Parameters(map)
 
     @staticmethod
-    def from_tuples(*tuples):
+    def from_tuples(*tuples: Any) -> 'Parameters':
         """
-        Creates a new Parameters object filled with provided key-value pairs called tuples.
+        Creates a new Parameters object filled with provided key-args pairs called tuples.
         Tuples parameters contain a sequence of key1, value1, key2, value2, ... pairs.
 
         :param tuples: the tuples to fill a new Parameters object.
@@ -234,7 +239,7 @@ class Parameters(AnyValueMap):
         return Parameters(map)
 
     @staticmethod
-    def merge_params(*parameters):
+    def merge_params(*parameters: Parameters) -> 'Parameters':
         """
         Merges two or more Parameters into one. The following Parameters override previously defined parameters.
 
@@ -246,7 +251,7 @@ class Parameters(AnyValueMap):
         return Parameters(map)
 
     @staticmethod
-    def from_json(json):
+    def from_json(json: str) -> 'Parameters':
         """
         Creates new Parameters from JSON object.
 
@@ -258,7 +263,7 @@ class Parameters(AnyValueMap):
         return Parameters(map)
 
     @staticmethod
-    def from_config(config):
+    def from_config(config: ConfigParams) -> 'Parameters':
         """
         Creates new Parameters from ConfigMap object.
 
@@ -267,11 +272,11 @@ class Parameters(AnyValueMap):
         :return: a new Parameters object.
         """
         result = Parameters()
-        
+
         if config is None or len(config) == 0:
             return result
-        
+
         for (key, value) in config.items():
             result.put(key, value)
-        
+
         return result
